import os
from pathlib import Path

import axipy
from PySide2.QtGui import QIcon, Qt
from PySide2.QtWidgets import QDialog, QMessageBox, QFileDialog
from axipy import data_manager, Attribute, Notifications

from PiketPoints.ui.addTools.helperTab import prepareTabSource, OutTabPiket
from PiketPoints.ui.addTools.helperUnit import factoryDistance
from PiketPoints.ui.helperUI import load_ui


def isTableVector(obj):
    if not isinstance(obj, axipy.Table):
        return False
    if not obj.is_spatial:
        return False
    try:
        schema_tab = obj.schema
        if schema_tab is None:
            return False
    except:
        return False
    return True
class DlgPiket(QDialog):
    def __init__(self, parent=None):
        super(DlgPiket, self).__init__(parent)
        self.ui=load_ui( "DlgPiket1.ui",parent)
        self.ui.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint)
        curent_path = str(Path(__file__).parent)
        path_icon = os.path.join(curent_path, "icons")
        path_save_png = os.path.join(path_icon, "saveas.png")
        self.ui.tb_saveas.setIcon(QIcon(path_save_png))
        self.ui.bt_run.clicked.connect(self.__run)
        self.ui.finished.connect(self.__closeDlg)
        self.ui.pb_close.clicked.connect(self.__close_dlg)
        self.ui.tb_saveas.clicked.connect(self.__select_save)
        self.ui.source_tabs.currentIndexChanged.connect(self.__changeTab)
        self.__initSource()
    def __initSource(self):
        tab_sel=data_manager.selection
        if tab_sel is not None:
            self.ui.source_tabs.addItem(tab_sel.name)
        for tab in data_manager.tables:
            if isTableVector(tab):
                self.ui.source_tabs.addItem(tab.name)

    def __changeTab(self):
        name_tab=self.ui.source_tabs.currentText()
        tab_cmb=data_manager.find(name_tab)
        path_tab=tab_cmb.properties['fileName']
        path_out_tab_cls=Path(path_tab)
        name_file=str(path_out_tab_cls.stem)
        path_out=str(path_out_tab_cls.parent.joinpath(name_file+"_piket.tab"))
        self.ui.ln_out_tab.setText(path_out)
        self.__fillNmameFields(name_tab)
        if tab_cmb.coordsystem.lat_lon:
            self.ui.comb_method.setCurrentIndex(1)
            self.ui.comb_method.setEnabled(False)
            return
        if tab_cmb.coordsystem.non_earth:
            self.ui.comb_method.setCurrentIndex(0)
            self.ui.comb_method.setEnabled(False)
            return
        self.ui.comb_method.setEnabled(True)
    def __fillNmameFields(self,name_tab):
        self.ui.fields_name.clear()
        self.ui.fields_name.addItem("*id_записи")
        for att in data_manager.find(name_tab).schema:
            self.ui.fields_name.addItem(att.name)
    def __run(self):
        name_tab=self.ui.source_tabs.currentText()
        tab_run = data_manager.find(name_tab)
        isPlanSchema = True
        if self.ui.comb_method.currentIndex() == 1:
            isPlanSchema = False

        tab_liner, all_distance_m = prepareTabSource(tab_run,isPlanSchema)
        if tab_liner is None:
            msg = QMessageBox(self.ui)
            msg.setWindowTitle("Создание точек пикетов")
            msg.setText("Выбранная таблица не содержит линий или полилиний")
            msg.setIcon(QMessageBox.Warning)

            msg.exec_()
            return
        distance_all = factoryDistance(all_distance_m, "m")
        str_unit=self.ui.cmb_unit.currentText()
        step = factoryDistance(float(self.ui.double_step.text()), str_unit)

        prefix=None
        if self.ui.ch_prefix.isChecked() and len(self.ui.ln_prefix.text().strip())>0:
            prefix=self.ui.ln_prefix.text().strip()
        if self.ui.fields_name.currentIndex()==0:
            att_parent_key=Attribute.integer("id_записи")
            att_parentkey={"parent":None,"out_att":att_parent_key}
        else:
            att_parent_key_tab=tab_run.schema[self.ui.fields_name.currentIndex()-1]
            att_parent_key=Attribute("id_записи",att_parent_key_tab.typedef)
            att_parentkey ={"parent":self.ui.fields_name,"out_att":att_parent_key}
        out_cs=None
        if not isPlanSchema:
            out_cs=tab_run.coordsystem
        path_out_tab=self.ui.ln_out_tab.text()
        try:
            self.ui.close()
            csl_piket = OutTabPiket(tab_liner, path_out_tab, distance_all, step, prefix, att_parentkey)
            procent_run = csl_piket.run_task(out_cs)
            Notifications.push('Информация', 'Таблица пикетов создана:'+path_out_tab, Notifications.Information)
        except Exception as ex:
            print(ex)
            pass
        tab_liner.close()
    def __select_save(self):
        ext_file = '*.tab'
        path_out=self.ui.ln_out_tab.text()
        name_save_file = QFileDialog.getSaveFileName(self, 'Сохранить результат', path_out, ext_file)
        if name_save_file is not None:
            self.ui.ln_out_tab.setText(name_save_file[0])
            self.ui.ln_out_tab.setCursorPosition(0)

    def __closeDlg(self, evnt):
        pass
    def exec_(self):
        self.ui.exec_()
    def __close_dlg(self):
        self.ui.close()
