import math

import axipy
from axipy import Table, CoordSystem, Schema, Attribute, Line, LineString, Point, Feature, provider_manager, \
    data_manager, LinearUnits, ProgressGuiFlags, ProgressSpecification, AxipyProgressHandler, task_manager

from PiketPoints.ui.addTools.helperPoints import PointsPiket
from PiketPoints.ui.addTools.helperUnit import Distance


def createAttribute(struct_att:dict)->Attribute:
    name=struct_att['name']
    type=struct_att['type']
    len_str=10
    try:
        len_str=struct_att['len']
    except:
        pass
    if type.lower()=='int':
        return Attribute.integer(name)
    if type.lower()=='float':
        return Attribute.float(name)
    if type.lower()=='str':
        return Attribute.string(name,len_str)
    return None
def createTab(path_out:str,description_struct:list,att_parent_field,cs:CoordSystem)->Table:
    schema = Schema(coordsystem=cs)
    schema.append(att_parent_field)
    for field in description_struct:
        att=createAttribute(field)
        if att is None:
            print("error parser struct")
            continue
        schema.append(att)
    #return axipy.provider_manager.tab.create_open(path_out,schema)
    definition = {
        'src': path_out,
        'schema': schema


    }
    return  provider_manager.create(definition)  # type: Table
class OutTabPiket:
    def __init__(self,tab_source ,path_out_tab,all_distance:Distance,step:Distance,prefix:str,field_key_att=None):
        self.__tab_source=tab_source
        self.__path_out=path_out_tab
        self.__step=step
        self.__prefix=prefix
        self.__all_distance=all_distance
        self.__parent_col_att=field_key_att
    def run(self,cs:CoordSystem=None):
        all_count_pikets=int(math.trunc(self.__all_distance.to_unit(LinearUnits.m)/self.__step.to_unit(LinearUnits.m)))
        cls_piket=PointsPiket()
        struct_table = []
        if self.__prefix is not None:
            struct_table.append({'name': "name", 'type': 'str', 'len': 20})
        struct_table.append({'name': "id", 'type': 'int', 'len': 20})
        struct_table.append({'name': "dist", 'type': 'float', 'len': 20})
        self.__tab_out = createTab(self.__path_out, struct_table,self.__parent_col_att['out_att'], self.__tab_source.coordsystem)
        for ft in self.__tab_source.items():
            geo=ft.geometry
            list_point=None
            if isinstance(geo,Line):
                list_point=[geo.begin,geo.end]
            if isinstance(geo,LineString):
                list_point=geo.points
            if list_point is None:
                continue
            points_piket=cls_piket.buildPikets(list_point,self.__step,cs)
            self.__save_piket(0,points_piket)
        self.__tab_out.commit()
        #self.__tab_out.close()
    def run_task(self,cs:CoordSystem=None):
        self.__cs_out=cs
        all_count_pikets = int(
            math.trunc(self.__all_distance.to_unit(LinearUnits.m) / self.__step.to_unit(LinearUnits.m)))
        spec = ProgressSpecification(description="Создание пикетов",
                                     flags=ProgressGuiFlags.NO_DELAY | ProgressGuiFlags.CANCELABLE)
        proc_run=task_manager.run_and_get(spec, self.__run, all_count_pikets)
        return proc_run
    def __run(self, ph: AxipyProgressHandler, all_count_pikets=None):
        cls_piket = PointsPiket(ph)
        ph.set_max_progress(all_count_pikets)
        struct_table = []
        struct_table.append({'name': "name", 'type': 'str', 'len': 20})
        struct_table.append({'name': "id", 'type': 'int', 'len': 20})
        struct_table.append({'name': "dist", 'type': 'float', 'len': 20})
        self.__tab_out = createTab(self.__path_out, struct_table, self.__parent_col_att['out_att'],self.__tab_source.coordsystem)
        index_record=0
        for ft in self.__tab_source.items():
            index_record=index_record+1
            geo = ft.geometry
            list_point = None
            if isinstance(geo, Line):
                list_point = [geo.begin, geo.end]
            if isinstance(geo, LineString):
                list_point = geo.points
            if list_point is None:
                continue
            points_piket = cls_piket.buildPikets(list_point, self.__step,self.__cs_out)
            if cls_piket.isCancel:
                break
            if self.__parent_col_att['parent'] is None:
                value_parent_key=index_record
            else:
                value_parent_key=ft[self.__parent_col_att['parent']]
            self.__save_piket(value_parent_key,points_piket)
        self.__tab_out.commit()
        return ph.is_canceled()
    def __save_piket(self,value_key,pikets):

        out_features=[]
        for i in range(len(pikets)):
            pt=pikets[i]
            att_ft={}
            att_ft[self.__parent_col_att['out_att'].name]=value_key
            if self.__prefix is not None:
                att_ft['name']=self.__prefix+str(i)
            att_ft['id']=i
            att_ft['dist']=i*self.__step.value
            geo_point=Point(pt.x,pt.y,self.__tab_source.coordsystem)
            ft=Feature(att_ft,geometry=geo_point)
            out_features.append(ft)
        self.__tab_out.insert(out_features)

def prepareTabSource(tab_source:Table,isPlanSchema=True):
    sql_liner_object="Select * from "+tab_source.name+' Where Str$(obj)=="Line" or Str$(obj)=="Polyline"'
    tab_liner=data_manager.query_hidden(sql_liner_object)
    if tab_liner.count()==0:
        return None,0
    if isPlanSchema:
        sql_distance='Select Sum(CartesianObjectLen(obj, "m")) as len From '+tab_liner.name
    else:
        sql_distance = 'Select Sum(SphericalObjectLen(obj, "m")) as len From '+tab_liner.name
    tab_dist=data_manager.query_hidden(sql_distance)
    if tab_dist is None:
        return None,0
    distance_m=list(tab_dist.items())[0]['len']
    tab_dist.close()
    return tab_liner,distance_m

